@@ -338,16 +338,7 @@ int
 boot_(UNUSED(Module m))
 {
 #ifdef USE_TERMINFO_MODULE
-# ifdef HAVE_SETUPTERM
-    int errret;
-
-    /*
-     * Just because we can't set up the terminal doesn't
-     * mean the modules hasn't booted---TERM may change,
-     * and it should be handled dynamically---so ignore errors here.
-     */
-    (void)setupterm((char *)0, 1, &errret);
-# endif
+    zsetupterm();
 #endif
 
     return 0;
@@ -357,6 +348,9 @@ boot_(UNUSED(Module m))
 int
 cleanup_(Module m)
 {
+#ifdef USE_TERMINFO_MODULE
+    zdeleteterm();
+#endif
     return setfeatureenables(m, &module_features, NULL);
 }
 
